@@ -1,43 +1,33 @@
 from __future__ import absolute_import
-import collections
-import shutil
-import json
 
+import logging
+from typing import (Any, Dict, Iterable, List,  # pylint: disable=unused-import
+                    MutableMapping, MutableSequence, Optional, Tuple, Union,
+                    cast)
+
+import rdflib
+import rdflib.namespace
+from rdflib import Graph, URIRef
+from rdflib.namespace import RDF, RDFS
 import six
 from six.moves import urllib
+from typing_extensions import Text  # pylint: disable=unused-import
+# move to a regular typing import when Python 3.3-3.6 is no longer supported
+
+from .ref_resolver import ContextType  # pylint: disable=unused-import
+from .utils import aslist, json_dumps
 
-import ruamel.yaml as yaml
-try:
-    from ruamel.yaml import CSafeLoader as SafeLoader
-except ImportError:
-    from ruamel.yaml import SafeLoader  # type: ignore
-import os
-import subprocess
-import copy
-import pprint
-import re
-import sys
-import rdflib
-from rdflib import Graph, URIRef
-import rdflib.namespace
-from rdflib.namespace import RDF, RDFS
-import logging
-from schema_salad.utils import aslist
-from typing import (cast, Any, Dict, Iterable, List, Optional, Text, Tuple,
-    Union)
-from .ref_resolver import Loader, ContextType
 
 _logger = logging.getLogger("salad")
 
 
-def pred(datatype,      # type: Dict[str, Union[Dict, str]]
+def pred(datatype,      # type: MutableMapping[Text, Union[Dict, Text]]
          field,         # type: Optional[Dict]
          name,          # type: str
          context,       # type: ContextType
          defaultBase,   # type: str
-         namespaces     # type: Dict[str, rdflib.namespace.Namespace]
-         ):
-    # type: (...) -> Union[Dict, Text]
+         namespaces     # type: Dict[Text, rdflib.namespace.Namespace]
+        ):  # type: (...) -> Union[Dict, Text]
     split = urllib.parse.urlsplit(name)
 
     vee = None  # type: Optional[Text]
@@ -53,7 +43,7 @@ def pred(datatype,      # type: Dict[str, Union[Dict, str]]
     v = None  # type: Optional[Dict]
 
     if field is not None and "jsonldPredicate" in field:
-        if isinstance(field["jsonldPredicate"], dict):
+        if isinstance(field["jsonldPredicate"], MutableMapping):
             v = {}
             for k, val in field["jsonldPredicate"].items():
                 v[("@" + k[1:] if k.startswith("_") else k)] = val
@@ -62,9 +52,9 @@ def pred(datatype,      # type: Dict[str, Union[Dict, str]]
         else:
             v = field["jsonldPredicate"]
     elif "jsonldPredicate" in datatype:
-        if isinstance(datatype["jsonldPredicate"], collections.Iterable):
+        if isinstance(datatype["jsonldPredicate"], Iterable):
             for d in datatype["jsonldPredicate"]:
-                if isinstance(d, dict):
+                if isinstance(d, MutableMapping):
                     if d["symbol"] == name:
                         v = d["predicate"]
                 else:
@@ -90,18 +80,20 @@ def pred(datatype,      # type: Dict[str, Union[Dict, str]]
     return ret
 
 
-def process_type(t,             # type: Dict[str, Any]
+def process_type(t,             # type: MutableMapping[Text, Any]
                  g,             # type: Graph
                  context,       # type: ContextType
                  defaultBase,   # type: str
-                 namespaces,    # type: Dict[str, rdflib.namespace.Namespace]
+                 namespaces,    # type: Dict[Text, rdflib.namespace.Namespace]
                  defaultPrefix  # type: str
-                 ):
-    # type: (...) -> None
-    if t["type"] == "record":
+                ):  # type: (...) -> None
+    if t["type"] not in ("record", "enum"):
+        return
+
+    if "name" in t:
         recordname = t["name"]
 
-        _logger.debug("Processing record %s\n", t)
+        _logger.debug("Processing %s %s\n", t.get("type"), t)
 
         classnode = URIRef(recordname)
         g.add((classnode, RDF.type, RDFS.Class))
@@ -127,13 +119,14 @@ def process_type(t,             # type: Dict[str, Any]
                       recordname, predicate, type(predicate))
         context[recordname] = predicate
 
+    if t["type"] == "record":
         for i in t.get("fields", []):
             fieldname = i["name"]
 
             _logger.debug("Processing field %s", i)
 
             v = pred(t, i, fieldname, context, defaultPrefix,
-                    namespaces)  # type: Union[Dict[Any, Any], Text, None]
+                     namespaces)  # type: Union[Dict[Any, Any], Text, None]
 
             if isinstance(v, six.string_types):
                 v = v if v[0] != "@" else None
@@ -152,7 +145,7 @@ def process_type(t,             # type: Dict[str, Any]
 
                 # TODO generate range from datatype.
 
-            if isinstance(i["type"], dict) and "name" in i["type"]:
+            if isinstance(i["type"], MutableMapping):
                 process_type(i["type"], g, context, defaultBase,
                              namespaces, defaultPrefix)
 
@@ -160,14 +153,14 @@ def process_type(t,             # type: Dict[str, Any]
             for e in aslist(t["extends"]):
                 g.add((classnode, RDFS.subClassOf, URIRef(e)))
     elif t["type"] == "enum":
-        _logger.debug("Processing enum %s", t["name"])
+        _logger.debug("Processing enum %s", t.get("name"))
 
         for i in t["symbols"]:
             pred(t, None, i, context, defaultBase, namespaces)
 
 
 def salad_to_jsonld_context(j, schema_ctx):
-    # type: (Iterable, Dict[str, Any]) -> Tuple[ContextType, Graph]
+    # type: (Iterable, MutableMapping[Text, Any]) -> Tuple[ContextType, Graph]
     context = {}  # type: ContextType
     namespaces = {}
     g = Graph()
@@ -184,7 +177,7 @@ def salad_to_jsonld_context(j, schema_ctx):
         defaultBase = ""
 
     for k, v in namespaces.items():
-        g.bind(k, v)
+        g.bind(str(k), v)
 
     for t in j:
         process_type(t, g, context, defaultBase, namespaces, defaultPrefix)
@@ -192,17 +185,16 @@ def salad_to_jsonld_context(j, schema_ctx):
     return (context, g)
 
 
-def fix_jsonld_ids(obj,     # type: Union[Dict[Text, Any], List[Dict[Text, Any]]]
-                   ids      # type: List[Text]
-                   ):
-    # type: (...) -> None
-    if isinstance(obj, dict):
+def fix_jsonld_ids(obj,  # type: Union[Dict[Text, Any], List[Dict[Text, Any]]]
+                   ids   # type: List[Text]
+                  ):  # type: (...) -> None
+    if isinstance(obj, MutableMapping):
         for i in ids:
             if i in obj:
                 obj["@id"] = obj[i]
         for v in obj.values():
             fix_jsonld_ids(v, ids)
-    if isinstance(obj, list):
+    if isinstance(obj, MutableSequence):
         for entry in obj:
             fix_jsonld_ids(entry, ids)
 
@@ -211,12 +203,11 @@ def makerdf(workflow,       # type: Text
             wf,             # type: Union[List[Dict[Text, Any]], Dict[Text, Any]]
             ctx,            # type: ContextType
             graph=None      # type: Graph
-            ):
-    # type: (...) -> Graph
+           ):  # type: (...) -> Graph
     prefixes = {}
     idfields = []
     for k, v in six.iteritems(ctx):
-        if isinstance(v, dict):
+        if isinstance(v, MutableMapping):
             url = v["@id"]
         else:
             url = v
@@ -234,13 +225,13 @@ def makerdf(workflow,       # type: Text
     else:
         g = graph
 
-    if isinstance(wf, list):
+    if isinstance(wf, MutableSequence):
         for w in wf:
             w["@context"] = ctx
-            g.parse(data=json.dumps(w), format='json-ld', publicID=str(workflow))
+            g.parse(data=json_dumps(w), format='json-ld', publicID=str(workflow))
     else:
         wf["@context"] = ctx
-        g.parse(data=json.dumps(wf), format='json-ld', publicID=str(workflow))
+        g.parse(data=json_dumps(wf), format='json-ld', publicID=str(workflow))
 
     # Bug in json-ld loader causes @id fields to be added to the graph
     for sub, pred, obj in g.triples((None, URIRef("@id"), None)):
