@@ -1,35 +1,32 @@
-from __future__ import print_function
-from __future__ import absolute_import
+"""Command line interface to schema-salad."""
+from __future__ import absolute_import, print_function
+
 import argparse
+import itertools
 import logging
-import sys
-import traceback
-import json
 import os
 import re
-import itertools
-
-import six
-from six.moves import urllib
+import sys
+from typing import (Any, Dict, List, Mapping, MutableSequence, Pattern, Tuple,
+                    Union, cast)
 
 import pkg_resources  # part of setuptools
+from rdflib.parser import Parser
+from rdflib.plugin import register
+from ruamel.yaml.comments import CommentedMap
+import six
+from six.moves import urllib
+from typing_extensions import Text  # pylint: disable=unused-import
+# move to a regular typing import when Python 3.3-3.6 is no longer supported
 
-from typing import Any, Dict, List, Union, Pattern, Text, Tuple, cast
-
-from rdflib import Graph, plugin
-from rdflib.serializer import Serializer
-
-from . import schema
-from . import jsonld_context
-from . import makedoc
-from . import validate
-from . import codegen
-from .sourceline import strip_dup_lineno
+from . import codegen, jsonld_context, schema, validate
 from .ref_resolver import Loader, file_uri
-_logger = logging.getLogger("salad")
+from .sourceline import strip_dup_lineno
+from .utils import json_dumps
+from .avro.schema import SchemaParseException
 
-from rdflib.plugin import register, Parser
 register('json-ld', Parser, 'rdflib_jsonld.parser', 'JsonLDParser')
+_logger = logging.getLogger("salad")
 
 
 def printrdf(workflow,  # type: str
@@ -39,8 +36,7 @@ def printrdf(workflow,  # type: str
              ):
     # type: (...) -> None
     g = jsonld_context.makerdf(workflow, wf, ctx)
-    print(g.serialize(format=sr))
-
+    print(g.serialize(format=sr, encoding='utf-8').decode('utf-8'))  # type: ignore
 
 def regex_chunk(lines, regex):
     # type: (List[str], Pattern[str]) -> List[List[str]]
@@ -62,21 +58,20 @@ def chunk_messages(message):  # type: (str) -> List[Tuple[int, str]]
     for chun in regex_chunk(message.splitlines(), file_regex):
         fst = chun[0]
         mat = file_regex.match(fst)
-        place = mat.group(1)
-        indent = len(mat.group(2))
+        if mat:
+            place = mat.group(1)
+            indent = len(mat.group(2))
 
-        lst = [mat.group(3)]+chun[1:]
-        if [x for x in lst if item_regex.match(x)]:
-            for item in regex_chunk(lst, item_regex):
-                msg = re.sub(item_regex, '', "\n".join(item))
-                arr.append((indent, place+' '+re.sub(r'[\n\s]+',
-                                                     ' ',
-                                                     msg)))
-        else:
-            msg = re.sub(item_regex, '', "\n".join(lst))
-            arr.append((indent, place+' '+re.sub(r'[\n\s]+',
-                                                 ' ',
-                                                 msg)))
+            lst = [mat.group(3)]+chun[1:]
+            if [x for x in lst if item_regex.match(x)]:
+                for item in regex_chunk(lst, item_regex):
+                    msg = re.sub(item_regex, '', "\n".join(item))
+                    arr.append((indent, place+' '+re.sub(
+                        r'[\n\s]+', ' ', msg)))
+            else:
+                msg = re.sub(item_regex, '', "\n".join(lst))
+                arr.append((indent, place+' '+re.sub(
+                    r'[\n\s]+', ' ', msg)))
     return arr
 
 
@@ -95,7 +90,7 @@ def to_one_line_messages(message):  # type: (str) -> str
 
 def reformat_yaml_exception_message(message):  # type: (str) -> str
     line_regex = re.compile(r'^\s+in "(.+)", line (\d+), column (\d+)$')
-    fname_regex = re.compile(r'^file://'+os.getcwd()+'/')
+    fname_regex = re.compile(r'^file://'+re.escape(os.getcwd())+'/')
     msgs = message.splitlines()
     ret = []
 
@@ -104,17 +99,21 @@ def reformat_yaml_exception_message(message):  # type: (str) -> str
         nblanks = 0
     elif len(msgs) == 4:
         c_msg = msgs[0]
-        c_file, c_line, c_column = line_regex.match(msgs[1]).groups()
-        c_file = re.sub(fname_regex, '', c_file)
-        ret.append("%s:%s:%s: %s" % (c_file, c_line, c_column, c_msg))
+        match = line_regex.match(msgs[1])
+        if match:
+            c_file, c_line, c_column = match.groups()
+            c_file = re.sub(fname_regex, '', c_file)
+            ret.append("%s:%s:%s: %s" % (c_file, c_line, c_column, c_msg))
 
         msgs = msgs[2:]
         nblanks = 2
 
     p_msg = msgs[0]
-    p_file, p_line, p_column = line_regex.match(msgs[1]).groups()
-    p_file = re.sub(fname_regex, '', p_file)
-    ret.append("%s:%s:%s:%s %s" % (p_file, p_line, p_column, ' '*nblanks, p_msg))
+    match = line_regex.match(msgs[1])
+    if match:
+        p_file, p_line, p_column = match.groups()
+        p_file = re.sub(fname_regex, '', p_file)
+        ret.append("%s:%s:%s:%s %s" % (p_file, p_line, p_column, ' '*nblanks, p_msg))
     return "\n".join(ret)
 
 
@@ -127,6 +126,10 @@ def main(argsl=None):  # type: (List[str]) -> int
                         help="Output RDF serialization format used by --print-rdf (one of turtle (default), n3, nt, xml)",
                         default="turtle")
 
+    parser.add_argument("--skip-schemas", action="store_true", default=False, help="If specified, ignore $schemas sections.")
+    parser.add_argument("--strict-foreign-properties", action="store_true", help="Strict checking of foreign properties",
+                        default=False)
+
     exgroup = parser.add_mutually_exclusive_group()
     exgroup.add_argument("--print-jsonld-context", action="store_true",
                          help="Print JSON-LD context for schema")
@@ -143,6 +146,10 @@ def main(argsl=None):  # type: (List[str]) -> int
         "--print-index", action="store_true", help="Print node index")
     exgroup.add_argument("--print-metadata",
                          action="store_true", help="Print document metadata")
+    exgroup.add_argument("--print-inheritance-dot",
+                         action="store_true", help="Print graphviz file of inheritance")
+    exgroup.add_argument("--print-fieldrefs-dot",
+                         action="store_true", help="Print graphviz file of field refs")
 
     exgroup.add_argument("--codegen", type=str, metavar="language", help="Generate classes in target language, currently supported: python")
 
@@ -214,29 +221,26 @@ def main(argsl=None):  # type: (List[str]) -> int
 
     # Optionally print the schema after ref resolution
     if not args.document and args.print_pre:
-        print(json.dumps(schema_doc, indent=4))
+        print(json_dumps(schema_doc, indent=4))
         return 0
 
     if not args.document and args.print_index:
-        print(json.dumps(list(metaschema_loader.idx.keys()), indent=4))
+        print(json_dumps(list(metaschema_loader.idx.keys()), indent=4))
         return 0
 
     # Validate the schema document against the metaschema
     try:
         schema.validate_doc(metaschema_names, schema_doc,
-                            metaschema_loader, args.strict,
-                            source=schema_metadata.get("name"))
+                            metaschema_loader, args.strict)
     except validate.ValidationException as e:
-        _logger.error("While validating schema `%s`:\n%s" %
-                      (args.schema, str(e)))
+        _logger.error("While validating schema `%s`:\n%s",
+                      args.schema, str(e))
         return 1
 
     # Get the json-ld context and RDFS representation from the schema
-    metactx = {}  # type: Dict[str, str]
-    if isinstance(schema_raw_doc, dict):
-        metactx = schema_raw_doc.get("$namespaces", {})
-        if "$base" in schema_raw_doc:
-            metactx["@base"] = schema_raw_doc["$base"]
+    metactx = schema.collect_namespaces(schema_metadata)
+    if "$base" in schema_metadata:
+        metactx["@base"] = schema_metadata["$base"]
     if schema_doc is not None:
         (schema_ctx, rdfs) = jsonld_context.salad_to_jsonld_context(
             schema_doc, metactx)
@@ -244,7 +248,7 @@ def main(argsl=None):  # type: (List[str]) -> int
         raise Exception("schema_doc is None??")
 
     # Create the loader that will be used to load the target document.
-    document_loader = Loader(schema_ctx)
+    document_loader = Loader(schema_ctx, skip_schemas=args.skip_schemas)
 
     if args.codegen:
         codegen.codegen(args.codegen, cast(List[Dict[Text, Any]], schema_doc),
@@ -253,39 +257,47 @@ def main(argsl=None):  # type: (List[str]) -> int
 
     # Make the Avro validation that will be used to validate the target
     # document
-    if isinstance(schema_doc, list):
-        (avsc_names, avsc_obj) = schema.make_avro_schema(
-            schema_doc, document_loader)
+    if isinstance(schema_doc, MutableSequence):
+        avsc_obj = schema.make_avro(schema_doc, document_loader)
+        try:
+            avsc_names = schema.make_avro_schema_from_avro(avsc_obj)
+        except SchemaParseException as err:
+            _logger.error("Schema `%s` error:\n%s", args.schema, err,
+                          exc_info=(
+                              (type(err), err, None) if args.debug else None))
+            if args.print_avro:
+                print(json_dumps(avsc_obj, indent=4))
+            return 1
     else:
         _logger.error("Schema `%s` must be a list.", args.schema)
         return 1
 
-    if isinstance(avsc_names, Exception):
-        _logger.error("Schema `%s` error:\n%s", args.schema,
-                      avsc_names, exc_info=((type(avsc_names), avsc_names,
-                                             None) if args.debug else None))
-        if args.print_avro:
-            print(json.dumps(avsc_obj, indent=4))
-        return 1
-
     # Optionally print Avro-compatible schema from schema
     if args.print_avro:
-        print(json.dumps(avsc_obj, indent=4))
+        print(json_dumps(avsc_obj, indent=4))
         return 0
 
     # Optionally print the json-ld context from the schema
     if args.print_jsonld_context:
         j = {"@context": schema_ctx}
-        print(json.dumps(j, indent=4, sort_keys=True))
+        print(json_dumps(j, indent=4, sort_keys=True))
         return 0
 
     # Optionally print the RDFS graph from the schema
     if args.print_rdfs:
-        print(rdfs.serialize(format=args.rdf_serializer))
+        print(rdfs.serialize(format=args.rdf_serializer).decode('utf-8'))  # type: ignore
         return 0
 
     if args.print_metadata and not args.document:
-        print(json.dumps(schema_metadata, indent=4))
+        print(json_dumps(schema_metadata, indent=4))
+        return 0
+
+    if args.print_inheritance_dot:
+        schema.print_inheritance(schema_doc, sys.stdout)
+        return 0
+
+    if args.print_fieldrefs_dot:
+        schema.print_fieldrefs(schema_doc, document_loader, sys.stdout)
         return 0
 
     # If no document specified, all done.
@@ -298,7 +310,7 @@ def main(argsl=None):  # type: (List[str]) -> int
         uri = args.document
         if not urllib.parse.urlparse(uri)[0]:
             doc = "file://" + os.path.abspath(uri)
-        document, doc_metadata = document_loader.resolve_ref(uri)
+        document, doc_metadata = document_loader.resolve_ref(uri, strict_foreign_properties=args.strict_foreign_properties)
     except validate.ValidationException as e:
         msg = strip_dup_lineno(six.text_type(e))
         msg = to_one_line_messages(str(msg)) if args.print_oneline else msg
@@ -315,17 +327,18 @@ def main(argsl=None):  # type: (List[str]) -> int
 
     # Optionally print the document after ref resolution
     if args.print_pre:
-        print(json.dumps(document, indent=4))
+        print(json_dumps(document, indent=4))
         return 0
 
     if args.print_index:
-        print(json.dumps(list(document_loader.idx.keys()), indent=4))
+        print(json_dumps(list(document_loader.idx.keys()), indent=4))
         return 0
 
-    # Validate the schema document against the metaschema
+    # Validate the user document against the schema
     try:
         schema.validate_doc(avsc_names, document,
-                            document_loader, args.strict)
+                            document_loader, args.strict,
+                            strict_foreign_properties=args.strict_foreign_properties)
     except validate.ValidationException as e:
         msg = to_one_line_messages(str(e)) if args.print_oneline else str(e)
         _logger.error("While validating document `%s`:\n%s" %
@@ -334,7 +347,7 @@ def main(argsl=None):  # type: (List[str]) -> int
 
     # Optionally convert the document to RDF
     if args.print_rdf:
-        if isinstance(document, (dict, list)):
+        if isinstance(document, (Mapping, MutableSequence)):
             printrdf(args.document, document, schema_ctx, args.rdf_serializer)
             return 0
         else:
@@ -342,7 +355,7 @@ def main(argsl=None):  # type: (List[str]) -> int
             return 1
 
     if args.print_metadata:
-        print(json.dumps(doc_metadata, indent=4))
+        print(json_dumps(doc_metadata, indent=4))
         return 0
 
     print("Document `%s` is valid" % args.document)
