@@ -1,17 +1,19 @@
 from __future__ import absolute_import
-import pprint
-import avro.schema
-from avro.schema import Schema
-import sys
-import re
+
 import logging
+import pprint
+from typing import Any, List, MutableMapping, MutableSequence, Set, Union
 
+from . import avro
+from .avro import schema  # pylint: disable=no-name-in-module, import-error
 import six
-from six.moves import urllib
-from six.moves import range
+from .avro.schema import \
+    Schema  # pylint: disable=unused-import, no-name-in-module, import-error
+from six.moves import range, urllib
+from typing_extensions import Text  # pylint: disable=unused-import
+# move to a regular typing import when Python 3.3-3.6 is no longer supported
 
-from typing import Any, List, Set, Union, Text
-from .sourceline import SourceLine, lineno_re, bullets, indent
+from .sourceline import SourceLine, bullets, indent, strip_dup_lineno
 
 _logger = logging.getLogger("salad")
 
@@ -25,11 +27,15 @@ class ClassValidationException(ValidationException):
 
 def validate(expected_schema,           # type: Schema
              datum,                     # type: Any
-             identifiers=[],            # type: List[Text]
+             identifiers=None,          # type: List[Text]
              strict=False,              # type: bool
-             foreign_properties=set()   # type: Set[Text]
+             foreign_properties=None    # type: Set[Text]
              ):
     # type: (...) -> bool
+    if not identifiers:
+        identifiers = []
+    if not foreign_properties:
+        foreign_properties = set()
     return validate_ex(
         expected_schema, datum, identifiers, strict=strict,
         foreign_properties=foreign_properties, raise_ex=False)
@@ -68,7 +74,8 @@ def validate_ex(expected_schema,                  # type: Schema
                 foreign_properties=None,          # type: Set[Text]
                 raise_ex=True,                    # type: bool
                 strict_foreign_properties=False,  # type: bool
-                logger=_logger                    # type: logging.Logger
+                logger=_logger,                   # type: logging.Logger
+                skip_foreign_properties=False     # type: bool
                 ):
     # type: (...) -> bool
     """Determine if a python datum is an instance of a schema."""
@@ -108,15 +115,6 @@ def validate_ex(expected_schema,                  # type: Schema
                 raise ValidationException(u"the value is not string")
             else:
                 return False
-    elif schema_type == 'bytes':
-        if isinstance(datum, str):
-            return True
-        else:
-            if raise_ex:
-                raise ValidationException(
-                    u"the value `%s` is not bytes" % vpformat(datum))
-            else:
-                return False
     elif schema_type == 'int':
         if (isinstance(datum, six.integer_types)
                 and INT_MIN_VALUE <= datum <= INT_MAX_VALUE):
@@ -172,7 +170,7 @@ def validate_ex(expected_schema,                  # type: Schema
             else:
                 return False
     elif isinstance(expected_schema, avro.schema.ArraySchema):
-        if isinstance(datum, list):
+        if isinstance(datum, MutableSequence):
             for i, d in enumerate(datum):
                 try:
                     sl = SourceLine(datum, i, ValidationException)
@@ -181,7 +179,8 @@ def validate_ex(expected_schema,                  # type: Schema
                                        foreign_properties=foreign_properties,
                                        raise_ex=raise_ex,
                                        strict_foreign_properties=strict_foreign_properties,
-                                       logger=logger):
+                                       logger=logger,
+                                       skip_foreign_properties=skip_foreign_properties):
                         return False
                 except ValidationException as v:
                     if raise_ex:
@@ -200,7 +199,7 @@ def validate_ex(expected_schema,                  # type: Schema
         for s in expected_schema.schemas:
             if validate_ex(s, datum, identifiers, strict=strict, raise_ex=False,
                            strict_foreign_properties=strict_foreign_properties,
-                           logger=logger):
+                           logger=logger, skip_foreign_properties=skip_foreign_properties):
                 return True
 
         if not raise_ex:
@@ -209,12 +208,14 @@ def validate_ex(expected_schema,                  # type: Schema
         errors = []  # type: List[Text]
         checked = []
         for s in expected_schema.schemas:
-            if isinstance(datum, list) and not isinstance(s, avro.schema.ArraySchema):
+            if isinstance(datum, MutableSequence) and not isinstance(s, avro.schema.ArraySchema):
                 continue
-            elif isinstance(datum, dict) and not isinstance(s, avro.schema.RecordSchema):
+            elif isinstance(datum, MutableMapping) and not isinstance(s, avro.schema.RecordSchema):
                 continue
-            elif (isinstance(datum, (bool, six.integer_types, float, six.string_types)) and  # type: ignore
-                  isinstance(s, (avro.schema.ArraySchema, avro.schema.RecordSchema))):
+            elif (isinstance(  # type: ignore
+                    datum, (bool, six.integer_types, float, six.string_types))
+                    and isinstance(s, (avro.schema.ArraySchema,
+                                       avro.schema.RecordSchema))):
                 continue
             elif datum is not None and s.type == "null":
                 continue
@@ -225,7 +226,7 @@ def validate_ex(expected_schema,                  # type: Schema
                             foreign_properties=foreign_properties,
                             raise_ex=True,
                             strict_foreign_properties=strict_foreign_properties,
-                            logger=logger)
+                            logger=logger, skip_foreign_properties=skip_foreign_properties)
             except ClassValidationException as e:
                 raise
             except ValidationException as e:
@@ -238,7 +239,7 @@ def validate_ex(expected_schema,                  # type: Schema
                 type(datum).__name__, friendly(expected_schema)))
 
     elif isinstance(expected_schema, avro.schema.RecordSchema):
-        if not isinstance(datum, dict):
+        if not isinstance(datum, MutableMapping):
             if raise_ex:
                 raise ValidationException(u"is not a dict")
             else:
@@ -282,7 +283,7 @@ def validate_ex(expected_schema,                  # type: Schema
                                    foreign_properties=foreign_properties,
                                    raise_ex=raise_ex,
                                    strict_foreign_properties=strict_foreign_properties,
-                                   logger=logger):
+                                   logger=logger, skip_foreign_properties=skip_foreign_properties):
                     return False
             except ValidationException as v:
                 if f.name not in datum:
@@ -300,25 +301,28 @@ def validate_ex(expected_schema,                  # type: Schema
                 sl = SourceLine(datum, d, six.text_type)
                 if d not in identifiers and d not in foreign_properties and d[0] not in ("@", "$"):
                     if (d not in identifiers and strict) and (
-                            d not in foreign_properties and strict_foreign_properties) and not raise_ex:
+                            d not in foreign_properties and strict_foreign_properties and not skip_foreign_properties) and not raise_ex:
                         return False
                     split = urllib.parse.urlsplit(d)
                     if split.scheme:
-                        err = sl.makeError(u"unrecognized extension field `%s`%s."
-                                           "  Did you include "
-                                           "a $schemas section?" % (
-                                               d, " and strict_foreign_properties is True" if strict_foreign_properties else ""))
-                        if strict_foreign_properties:
-                            errors.append(err)
-                        else:
-                            logger.warn(err)
+                        if not skip_foreign_properties:
+                            err = sl.makeError(u"unrecognized extension field `%s`%s.%s"
+                                               % (d,
+                                                  " and strict_foreign_properties checking is enabled"
+                                                  if strict_foreign_properties else "",
+                                                  "\nForeign properties from $schemas:\n  %s" % "\n  ".join(sorted(foreign_properties))
+                                                  if len(foreign_properties) > 0 else ""))
+                            if strict_foreign_properties:
+                                errors.append(err)
+                            elif len(foreign_properties) > 0:
+                                logger.warning(strip_dup_lineno(err))
                     else:
                         err = sl.makeError(u"invalid field `%s`, expected one of: %s" % (
                             d, ", ".join("'%s'" % fn.name for fn in expected_schema.fields)))
                         if strict:
                             errors.append(err)
                         else:
-                            logger.warn(err)
+                            logger.warning(err)
 
         if bool(errors):
             if raise_ex:
